import { useState, useEffect } from 'react';
import { csv, scaleLinear, extent, scaleOrdinal } from 'd3';

export const VizContainer = () => {
  const csvUrl =
    'https://gist.githubusercontent.com/curran/a08a1080b88344b0c8a7/raw/639388c2cbc2120a14dcf466e85730eb8be498bb/iris.csv';

  const width = 960;
  const menuHeight = 75;
  const height = 600 - menuHeight;
  const margin = {
    top: 20,
    right: 200,
    bottom: 100,
    left: 40,
  };

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.right - margin.left;

  const radius = 10;

  const [data, setData] = useState(null);
  const [hoveredValue, setHoveredValue] = useState(null);

  const attributes = [
    { value: 'sepal_length', label: 'Sepal Length' },
    { value: 'sepal_width', label: 'Sepal Width' },
    { value: 'petal_length', label: 'Petal Length' },
    { value: 'petal_width', label: 'Petal Width' },
    { value: 'species', label: 'Species' },
  ];

  useEffect(() => {
    const row = (d) => {
      d.sepal_length = +d.sepal_length;
      d.sepal_width = +d.sepal_width;
      d.petal_length = +d.petal_length;
      d.petal_width = +d.petal_width;
      return d;
    };
    csv(csvUrl, row).then((data) => {
      setData(data);
    });
  }, []);

  const initialXAttribute = 'petal_width';
  const [xAttribute, setXAttribute] = useState(
    initialXAttribute
  );
  const xValue = (d) => d[xAttribute];

  const initialYAttribute = 'petal_length';
  const [yAttribute, setYAttribute] = useState(
    initialYAttribute
  );
  const yValue = (d) => d[yAttribute];

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const colorValue = (d) => d.species;
  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(['#000000', '#999999', '#dedede']);
  const colorLegendLabel = 'Species';
  
  const filteredData = data.filter(d => hoveredValue === colorValue(d));
  console.log("first");
  console.log(filteredData);

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight])
    .nice();

  const AxisBottom = () =>
    xScale.ticks().map((tickValue) => (
      <g
        className="ticks"
        transform={`translate(${xScale(tickValue)},0)`}
      >
        <line y2={innerHeight} />
      </g>
    ));

  const AxisLeft = () =>
    yScale.ticks().map((tickValue) => (
      <g
        className="ticks"
        transform={`translate(0, ${yScale(tickValue)})`}
      >
        <line x2={innerWidth} />
      </g>
    ));

  const MyMarks = ({
    data,
    radius,
    xScale,
    xValue,
    yScale,
    yValue,
    colorScale,
    colorValue,
  }) =>
    data.map((d) => (
      <circle
        cx={xScale(xValue(d))}
        cy={yScale(yValue(d))}
        r={radius}
        fill={colorScale(colorValue(d))}
        className={'circles'}
      />
    ));

  const Dropdown = ({
    options,
    id,
    selectedValue,
    onSelectedValueChange,
  }) => (
    <select
      id={id}
      onChange={(event) =>
        onSelectedValueChange(event.target.value)
      }
    >
      {options.map(({ value, label }) => (
        <option
          value={value}
          selected={value === selectedValue}
        >
          {label}
        </option>
      ))}
    </select>
  );

  const ColorLegend = ({
    colorScale,
    radius,
    itemOffest,
    textOffset,
    onHover,
    hoveredValue
  }) =>
    colorScale.domain().map((domainValue, i) => (
      <g
        className="tick"
        transform={`translate(0,${i * itemOffest})`}
        onMouseEnter={() => onHover(domainValue)}
        onMouseOut= {() => onHover(null)}
        opacity = {hoveredValue && domainValue !== hoveredValue? 0.2 : 1}
      >
        <circle fill={colorScale(domainValue)} r={radius} />
        <text x={textOffset} dy=".32em">
          {domainValue}
        </text>
      </g>
    ));

  console.log(colorScale.domain());

  return (
    <>
      <label for="x-select">X;;;;;;;;;;;;;;;;;;;:</label>
      <Dropdown
        options={attributes}
        id="x-select"
        selectedValue={xAttribute}
        onSelectedValueChange={setXAttribute}
      />
      <label for="y-select">X;;;;;;;;;;;;;;;;;;;:</label>
      <Dropdown
        options={attributes}
        id="y-select"
        selectedValue={yAttribute}
        onSelectedValueChange={setYAttribute}
      />

      <svg height={height} width={width}>
        <g
          transform={`translate(${margin.left},${margin.top})`}
        >
          <g
            transform={`translate(${innerWidth + 40}, 20)`}
          >
            <text
              x={20}
              y={-25}
              className="axis-label"
              textAnchor="middle"
            >
              {colorLegendLabel}
            </text>
            <ColorLegend
              colorScale={colorScale}
              radius={10}
              itemOffest={25}
              textOffset={20}
              onHover={setHoveredValue}
              hoveredValue={hoveredValue}
            />
          </g>
          <AxisBottom />
          <AxisLeft />
          <g opacity = {hoveredValue? 0.2 : 1}>
           <MyMarks
            data={data}
            radius={radius}
            xScale={xScale}
            xValue={xValue}
            yScale={yScale}
            yValue={yValue}
            colorScale={colorScale}
            colorValue={colorValue}
             />
          </g>
          <MyMarks
            data={filteredData}
            radius={radius}
            xScale={xScale}
            xValue={xValue}
            yScale={yScale}
            yValue={yValue}
            colorScale={colorScale}
            colorValue={colorValue}
          />
        </g>
      </svg>
    </>
  );
};
